module Acme::Client::Resources; end

require 'acme/client/resources/directory'
require 'acme/client/resources/account'
require 'acme/client/resources/order'
require 'acme/client/resources/authorization'
require 'acme/client/resources/challenges'
